@@ -11,10 +11,10 @@ setClass("GroupPars",
                         itemtrace='matrix',
                         lbound='numeric',
                         ubound='numeric',
-                        n.prior.mu='numeric',
-                        n.prior.sd='numeric',
-                        b.prior.alpha='numeric',
-                        b.prior.beta='numeric')
+                        any.prior='logical',
+                        prior.type='character',
+                        prior_1='numeric',
+                        prior_2='numeric')
 )
 
 setClass("RandomPars",
@@ -31,10 +31,10 @@ setClass("RandomPars",
                         cand.t.var='numeric',
                         drawvals='matrix',
                         mtch='numeric',
-                        n.prior.mu='numeric',
-                        n.prior.sd='numeric',
-                        b.prior.alpha='numeric',
-                        b.prior.beta='numeric')
+                        any.prior='logical',
+                        prior.type='character',
+                        prior_1='numeric',
+                        prior_2='numeric')
 )
 
 setClass("AllItemsClass",
@@ -55,11 +55,9 @@ setClass("AllItemsClass",
                         lbound='numeric',
                         ubound='numeric',
                         any.prior='logical',
-                        n.prior.mu='numeric',
-                        n.prior.sd='numeric',
-                        b.prior.alpha='numeric',
-                        b.prior.beta='numeric',
-                        D='numeric', #scaling correction
+                        prior.type='character',
+                        prior_1='numeric',                        
+                        prior_2='numeric',
                         'VIRTUAL')
 )
 
@@ -96,8 +94,6 @@ setClass('custom', contains = 'AllItemsClass',
 
 setGeneric('ProbTrace', function(x, Theta, ...) standardGeneric("ProbTrace"))
 
-setGeneric('LogLik', function(x, Theta, ...) standardGeneric("LogLik"))
-
 setGeneric('ExtractLambdas', function(x) standardGeneric("ExtractLambdas"))
 
 setGeneric('ExtractZetas', function(x) standardGeneric("ExtractZetas"))
