@@ -1,4 +1,4 @@
-EM.group <- function(pars, constrain, PrepList, list, Theta)
+EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
 {
     verbose <- list$verbose
     nfact <- list$nfact
@@ -14,7 +14,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
     J <- length(itemloc) - 1L
     nfullpars <- 0
     estpars <- c()
-    gfulldata <- gtheta0 <- gstructgrouppars <- gitemtrace <- vector('list', ngroups)
+    prodlist <- PrepList[[1L]]$prodlist
+    gfulldata <- gtheta0 <- gstructgrouppars <- vector('list', ngroups)
     for(g in 1L:ngroups){
         gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
         gfulldata[[g]] <- PrepList[[g]]$fulldata
@@ -67,7 +68,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
              paste(paste0(redindex[diag(L)[!estpars] > 0]), ''), ' but should only be applied to
                  estimated parameters. Please fix!')
     }
-    Prior <- prior <- gstructgrouppars <- rlist <- r <- list()
+    Prior <- prior <- Priorbetween <- gstructgrouppars <- rlist <- r <- list()
     #make sure constrained pars are equal
     tmp <- rowSums(L)
     tmp[tmp == 0] <- 1L
@@ -95,36 +96,38 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
        for(i in 1L:length(constrain))
            est[constrain[[i]][-1L]] <- FALSE
     EMhistory <- matrix(NA, NCYCLES+1L, length(longpars))
-    EMhistory[1L,] <- longpars
-    any.prior <- FALSE
+    EMhistory[1L,] <- longpars    
+    gTheta <- vector('list', ngroups)
     for(g in 1L:ngroups){
-        for(i in 1L:J){
-            pars[[g]][[i]]@any.prior <- any(!is.nan(pars[[g]][[i]]@n.prior.mu)) || 
-                any(!is.nan(pars[[g]][[i]]@b.prior.alpha)) 
-            if(pars[[g]][[i]]@any.prior) any.prior <- TRUE
+        gTheta[[g]] <- Theta 
+        if(length(prodlist) > 0L)
+            gTheta[[g]] <- prodterms(gTheta[[g]],prodlist)
+        if(BFACTOR){
+            Thetabetween <- thetaComb(theta=theta, nfact=nfact-ncol(sitems))
+            prior[[g]] <- dnorm(theta, 0, 1)
+            prior[[g]] <- prior[[g]]/sum(prior[[g]])  
         }
     }
-    gTheta <- vector('list', ngroups)
-    for(g in 1L:ngroups) gTheta[[g]] <- Theta
-    warn <- FALSE
+    preMstep.longpars2 <- preMstep.longpars <- longpars
+    accel <- 0
     
     #EM
     for (cycles in 1L:NCYCLES){
         #priors
         for(g in 1L:ngroups){            
-            gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
-            gTheta[[g]] <- Theta %*% chol(gstructgrouppars[[g]]$gcov) + gstructgrouppars[[g]]$gmeans
-            gitemtrace[[g]] <- computeItemtrace(pars=pars[[g]], Theta=gTheta[[g]], itemloc=itemloc)            
-            
+            gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])            
             if(BFACTOR){
-                prior[[g]] <- dnorm(theta, 0, 1)
-                prior[[g]] <- prior[[g]]/sum(prior[[g]])                
-                Prior[[g]] <- apply(expand.grid(prior[[g]], prior[[g]]), 1, prod)                
+                sel <- 1L:(nfact-ncol(sitems))
+                Priorbetween[[g]] <- mvtnorm::dmvnorm(Thetabetween,
+                                        gstructgrouppars[[g]]$gmeans[sel],
+                                        gstructgrouppars[[g]]$gcov[sel,sel,drop=FALSE])                
+                Priorbetween[[g]] <- Priorbetween[[g]]/sum(Priorbetween[[g]]) 
+                Prior[[g]] <- apply(expand.grid(Priorbetween[[g]], prior[[g]]), 1, prod)                
                 next
             }
-            Prior[[g]] <- mvtnorm::dmvnorm(gTheta[[g]],gstructgrouppars[[g]]$gmeans,
+            Prior[[g]] <- mvtnorm::dmvnorm(gTheta[[g]][ ,1L:nfact,drop=FALSE],gstructgrouppars[[g]]$gmeans,
                                            gstructgrouppars[[g]]$gcov)
-            Prior[[g]] <- Prior[[g]]/sum(Prior[[g]])
+            Prior[[g]] <- Prior[[g]]/sum(Prior[[g]])                                              
         }
         #Estep
         lastLL <- LL
@@ -132,29 +135,36 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
         for(g in 1L:ngroups){
             if(BFACTOR){
                 rlist[[g]] <- Estep.bfactor(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata,
-                                            Theta=gTheta[[g]], prior=prior[[g]],
-                                            specific=specific, sitems=sitems,
-                                            itemloc=itemloc, itemtrace=gitemtrace[[g]])
+                                            Theta=gTheta[[g]], prior=prior[[g]], Prior=Prior[[g]],
+                                            Priorbetween=Priorbetween[[g]], specific=specific, sitems=sitems,
+                                            itemloc=itemloc)
             } else {
                 rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata,
-                                         Theta=gTheta[[g]], prior=Prior[[g]], itemloc=itemloc,
-                                         itemtrace=gitemtrace[[g]])
+                                         Theta=gTheta[[g]], prior=Prior[[g]], itemloc=itemloc)
             }
             LL <- LL + sum(r[[g]]*log(rlist[[g]]$expected))
-        }                
-        if(cycles > 3L && lastLL > LL) warn <- TRUE
+        }
         for(g in 1L:ngroups){
             for(i in 1L:J){
                 tmp <- c(itemloc[i]:(itemloc[i+1L] - 1L))
                 pars[[g]][[i]]@rs <- rlist[[g]]$r1[, tmp]
             }
         }
+        preMstep.longpars2 <- preMstep.longpars
         preMstep.longpars <- longpars
         if(all(!est)) break        
         longpars <- Mstep(pars=pars, est=est, longpars=longpars, ngroups=ngroups, J=J,
-                      gTheta=gTheta, Prior=Prior, BFACTOR=BFACTOR, itemloc=itemloc,
+                      gTheta=gTheta, itemloc=itemloc, Prior=Prior,
                       PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND,
-                      constrain=constrain, TOL=MSTEPTOL)
+                      constrain=constrain, cycle=cycles, PROBTRACE=PROBTRACE, DERIV=DERIV)                   
+        if(list$accelerate && cycles > 10L && cycles %% 3 == 0L){            
+            dX2 <- preMstep.longpars - preMstep.longpars2
+            dX <- longpars - preMstep.longpars
+            d2X2 <- dX - dX2
+            accel <- 1 - sqrt((dX2 %*% dX2) / (d2X2 %*% d2X2))
+            if(accel < -4) accel <- -4
+            longpars <- (1 - accel) * longpars + accel * preMstep.longpars
+        }        
         pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
         EMhistory[cycles+1L,] <- longpars
         if(verbose)
@@ -164,9 +174,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
         if(cycles > 3L && all(abs(preMstep.longpars - longpars) < TOL))  break
     } #END EM
     if(cycles == NCYCLES)
-        message('EM iterations terminated after ', cycles, ' iterations.')    
-    if(warn && !any.prior)
-        warning('EM likelihood was not strictly increasing. Model is probably not identified or is unstable')
+        message('EM iterations terminated after ', cycles, ' iterations.')        
     infological <- estpars & !redun_constr
     correction <- numeric(length(estpars[estpars & !redun_constr]))
     names(correction) <- names(estpars[estpars & !redun_constr])
@@ -176,8 +184,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta)
         ind1 <- 1L
         for(group in 1L:ngroups){
             for (i in 1L:J){
-                deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE, prior=Prior[[group]],
-                               estHess=TRUE, BFACTOR=BFACTOR)
+                deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE, 
+                               estHess=TRUE)
                 ind2 <- ind1 + length(deriv$grad) - 1L
                 h[ind1:ind2, ind1:ind2] <- pars[[group]][[i]]@hessian <- deriv$hess
                 ind1 <- ind2 + 1L
@@ -221,29 +229,36 @@ Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, itemtrace=NULL, der
 }
 
 # Estep for bfactor
-Estep.bfactor <- function(pars, tabdata, Theta, prior, specific, sitems, itemloc, itemtrace=NULL)
+Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, specific, 
+                          sitems, itemloc, itemtrace=NULL)
 {
     J <- length(itemloc) - 1L
     r <- tabdata[ ,ncol(tabdata)]
     X <- tabdata[ ,1L:(ncol(tabdata) - 1L)]
     if(is.null(itemtrace))
         itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
-    retlist <- .Call("Estepbfactor", itemtrace, prior, X, r, sitems)
+    retlist <- .Call("Estepbfactor", itemtrace, prior, Priorbetween, X, r, sitems)
     r1 <- matrix(0, nrow(Theta), ncol(X))
     for (i in 1L:J){
         r1[ ,itemloc[i]:(itemloc[i+1L]-1L)] <-
             retlist$r1[ ,itemloc[i]:(itemloc[i+1L]-1L) + (specific[i] - 1L)*ncol(X) ]
     }
+    r1 <- r1 * Prior 
     return(list(r1=r1, expected=retlist$expected))
 }
 
-Mstep <- function(pars, est, longpars, ngroups, J, gTheta, Prior, BFACTOR, itemloc, PrepList, L,
-                  UBOUND, LBOUND, constrain, TOL){
+Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L,
+                  UBOUND, LBOUND, constrain, cycle, PROBTRACE, DERIV, Prior){
     p <- longpars[est]
-    opt <- optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS', 
-                 est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
-                 PrepList=PrepList, Prior=Prior, L=L, BFACTOR=BFACTOR, constrain=constrain,
-                 UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc, lower=LBOUND[est], upper=UBOUND[est])
+    opt <- try(optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS-B', 
+                     control=list(maxit=ifelse(cycle > 10L, 10L, 5L), fnscale = -1L),  
+                     PROBTRACE=PROBTRACE, DERIV=DERIV,
+                     est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
+                     PrepList=PrepList, L=L, constrain=constrain,
+                     UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc, lower=LBOUND[est], upper=UBOUND[est]),
+            silent=TRUE)
+    if(is(opt, 'try-error'))
+        stop(opt)
     longpars[est] <- opt$par
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
@@ -260,33 +275,44 @@ Mstep <- function(pars, est, longpars, ngroups, J, gTheta, Prior, BFACTOR, iteml
     return(longpars)
 }
 
-Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, Prior, L, BFACTOR,
-                     constrain, LBOUND, UBOUND, itemloc){
+Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
+                     constrain, LBOUND, UBOUND, itemloc, PROBTRACE, DERIV){
     longpars[est] <- p
     if(length(constrain) > 0L)
        for(i in 1L:length(constrain))
            longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
     pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
     LLs <- numeric(J)
-    LL <- 0    
+    LL <- 0        
     for(g in 1L:ngroups){                
         for(i in 1L:J)
-            LLs[i] <- LogLikMstep(pars[[g]][[i]], Theta=gTheta[[g]], EM=BFACTOR, prior=Prior[[g]])
+            LLs[i] <- LogLikMstep(pars[[g]][[i]], Theta=gTheta[[g]],
+                                  PROBTRACE=PROBTRACE[[g]][[i]])
         LL <- LL + sum(LLs)
     }
-    LL
+    if(is.nan(LL)){
+        for(g in 1L:ngroups){                
+            for(i in 1L:J){
+                if(is.nan(LogLikMstep(pars[[g]][[i]], Theta=gTheta[[g]],
+                                      PROBTRACE=PROBTRACE[[g]][[i]])))
+                    stop('M-step optimizer did not converge. 
+                         Item ', i, ' in group ', g, ' had the following extreme parameters:\n', 
+                         paste0(paste0('  ', names(pars[[g]][[i]]@est), ' = '), 
+                                round(pars[[g]][[i]]@par), 4))
+            }
+        }        
+    } else return(LL)    
 }
 
-LogLikMstep <- function(x, Theta, EM=FALSE, prior=NULL){
-    itemtrace <- ProbTrace(x=x, Theta=Theta)
-    if(EM) LL <- (-1) * sum(x@rs * log(itemtrace) * prior)
-    else LL <- (-1) * sum(x@rs * log(itemtrace))
+LogLikMstep <- function(x, Theta, PROBTRACE){
+    log_itemtrace <- log(PROBTRACE(x=x, Theta=Theta))
+    LL <- sum(x@rs * log_itemtrace)
     if(x@any.prior) LL <- LL.Priors(x=x, LL=LL)
     return(LL)
 }
 
-Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, Prior, L, BFACTOR,
-                       constrain, LBOUND, UBOUND, itemloc){
+Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
+                       constrain, LBOUND, UBOUND, itemloc, PROBTRACE, DERIV){
     longpars[est] <- p
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
@@ -296,8 +322,7 @@ Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, Pri
     ind1 <- 1L
     for(group in 1L:ngroups){
         for (i in 1L:J){
-            deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[group]], EM=TRUE, BFACTOR=BFACTOR,
-                           prior=Prior[[group]])
+            deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gTheta[[group]], EM=TRUE)
             ind2 <- ind1 + length(deriv$grad) - 1L
             g[ind1:ind2] <- deriv$grad
             ind1 <- ind2 + 1L
@@ -308,6 +333,5 @@ Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, Pri
         ind1 <- ind2 + 1L
     }
     grad <- g %*% L
-    grad <- grad[est]
-    -grad
+    return(grad[est])
 }
