@@ -132,7 +132,7 @@ fitIndices <- function(obj, prompt = TRUE){
             tmp <- c(itemloc[i]:(itemloc[i+1L] - 1L))
             pars[[i]]@rs <- rlist$r1[, tmp]
             pars[[i]]@itemtrace <- rlist$itemtrace[, tmp]
-            dx <- Deriv(pars[[i]], Theta=Theta, EM = TRUE, prior=Prior, estHess=FALSE)$grad
+            dx <- Deriv(pars[[i]], Theta=Theta, EM = TRUE, estHess=FALSE)$grad
             DX <- c(DX, dx[pars[[i]]@est])
         }
         if(is.null(delta)) delta <- matrix(NA, nrow(tabdata), length(DX), byrow = TRUE)
