@@ -5,7 +5,8 @@
 #' prior distribution. Will return either a table with the computed scores and standard errors, 
 #' the original data matrix with scores appended to the rightmost column, or the scores only. By
 #' default the latent means are set to be 0 for each factor, and the covariance matrix is set to the 
-#' identity matrix, though these can be overwritten.
+#' identity matrix, though these can be overwritten. Iterative estimation methods can be estimated 
+#' in parallel to decrease estimation times if a \code{\link{mirtCluster}} object is available.
 #'
 #'
 #' @aliases fscores
@@ -27,8 +28,8 @@
 #' from the computed mirt object will be used
 #' @param degrees the degrees argument to be passed to \code{\link{iteminfo}}, only necessary for
 #' multidimensional models when \code{method = 'WLE'}
-#' @param response.vector an optional argument used to calculate the factor scores and standard errors
-#' for a given response vector that may or may not have been in the original dataset
+#' @param response.pattern an optional argument used to calculate the factor scores and standard errors
+#' for a given response vector or matrix
 #' @param returnER logical; return empirical reliability estimate as a numeric value?
 #' @param verbose logical; print verbose output messages?
 #' @param scores.only logical; return only the factor scores (only applicable when \code{full.scores = TRUE})
@@ -58,20 +59,26 @@
 #' head(scores)
 #'
 #' #calculate MAP for a given response vector
-#' fscores(mod, method='MAP', response.vector = c(1,2,3,4))
+#' fscores(mod, method='MAP', response.pattern = c(1,2,3,4))
+#' #or matrix 
+#' fscores(mod, method='MAP', response.pattern = rbind(c(1,2,3,4), c(2,2,1,3)))
 #' 
 #' #use custom latent variable properties (diffuse prior for MAP is very close to ML)
 #' fscores(mod, method='MAP', cov = matrix(1000))
 #' fscores(mod, method='ML')
+#' 
+#' #WLE estimation, run in parallel using available cores
+#' mirtCluster()
+#' fscores(mod, method='WLE')
+#' 
 #'   }
-#'
 fscores <- function(object, rotate = '', full.scores = FALSE, method = "EAP",
-                    quadpts = NULL, response.vector = NULL, degrees = NULL,
+                    quadpts = NULL, response.pattern = NULL, degrees = NULL,
                     returnER = FALSE, mean = NULL, cov = NULL, verbose = TRUE,
                     scores.only = FALSE)
 {
     ret <- fscores.internal(object=object, rotate=rotate, full.scores=full.scores, method=method,
-                            quadpts=quadpts, response.vector=response.vector, degrees=degrees,
+                            quadpts=quadpts, response.pattern=response.pattern, degrees=degrees,
                             verbose=verbose, returnER=returnER, gmean=mean, gcov=cov, 
                             scores.only=scores.only)
     ret
