@@ -1,9 +1,9 @@
 #' Imputing plausible data for missing values
-#' 
-#' Given an estimated model from any of mirt's model fitting functions and an estimate of the latent trait, 
-#' impute plausible missing data values. Returns the original data in a \code{data.frame} 
+#'
+#' Given an estimated model from any of mirt's model fitting functions and an estimate of the latent trait,
+#' impute plausible missing data values. Returns the original data in a \code{data.frame}
 #' without any NA values.
-#' 
+#'
 #'
 #' @aliases imputeMissing
 #' @param x an estimated model x from the mirt package
@@ -11,7 +11,7 @@
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords impute data
 #' @export imputeMissing
-#' @examples 
+#' @examples
 #' \dontrun{
 #' dat <- expand.table(LSAT7)
 #' (original <- mirt(dat, 1))
@@ -25,40 +25,39 @@
 #' #re-estimate imputed dataset (good to do this multiple times and average over)
 #' fulldata <- imputeMissing(mod, scores[,'F1', drop = FALSE])
 #' (fullmod <- mirt(fulldata, 1))
-#' 
-#' 
+#'
+#'
 #' }
 imputeMissing <- function(x, Theta){    
-    set.seed(proc.time()[3])    
     if(is(x, 'MixedClass'))
-        stop('mixedmirt xs not yet supported')       
+        stop('mixedmirt xs not yet supported')
     if(is(x, 'MultipleGroupClass')){
         cmods <- x@cmods
-        group <- x@group                
+        group <- x@group
         data <- x@data
-        for(i in 1:length(cmods)){
+        for(i in 1L:length(cmods)){
             sel <- group == x@groupNames[i]
             Thetatmp <- Theta[sel, , drop = FALSE]
-            data[sel, ] <- imputeMissing(cmods[[i]], Thetatmp)            
+            data[sel, ] <- imputeMissing(cmods[[i]], Thetatmp)
         }
         return(data)
     }
-    pars <- x@pars        
-    K <- x@K        
-    J <- length(K)                
+    pars <- x@pars
+    K <- x@K
+    J <- length(K)
     data <- x@data
     N <- nrow(data)
-    Nind <- 1:N
-    for (i in 1:J){
+    Nind <- 1L:N
+    for (i in 1L:J){
         if(!any(is.na(data[,i]))) next
-        P <- ProbTrace(x=pars[[i]], Theta=Theta)        
+        P <- ProbTrace(x=pars[[i]], Theta=Theta)
         NAind <- Nind[is.na(data[,i])]
-        for(j in 1:length(NAind)){            
-            sampl <- sample(1:ncol(P), 1, prob = P[NAind[j], , drop = FALSE])
-            if(any(class(pars[[i]]) %in% c('dich', 'gpcm', 'partcomp'))) 
-                sampl <- sampl - 1 
-            data[NAind[j], i] <- sampl            
-        }        
+        for(j in 1L:length(NAind)){
+            sampl <- sample(1L:ncol(P), 1, prob = P[NAind[j], , drop = FALSE])
+            if(any(class(pars[[i]]) %in% c('dich', 'gpcm', 'partcomp', 'rsm')))
+                sampl <- sampl - 1L
+            data[NAind[j], i] <- sampl
+        }
     }
     return(data)
 }
