@@ -143,7 +143,7 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
         if(all(x@itemtype %in% c('Rasch', 'rsm', 'gpcm'))){
             oneslopes <- rep(FALSE, length(x@itemtype))
             for(i in 1L:length(x@itemtype))
-                oneslopes[i] <- closeEnough((x@pars[[i]]@par[1L] * x@pars[[1L]]@D), 1-1e-10, 1+1e-10)
+                oneslopes[i] <- closeEnough(x@pars[[i]]@par[1L], 1-1e-10, 1+1e-10)
             if(all(oneslopes)){
                 attr(x, 'inoutfitreturn') <- TRUE
                 pf <- personfit(x, method=method, sc=sc)
