@@ -7,7 +7,7 @@
 #'
 #' By default the estimation in \code{multipleGroup} assumes that the models are maximally
 #' independent, and therefore could initially be performed by sub-setting the data and running identical
-#' models with \code{confmirt} or \code{mirt} and aggregating the results (e.g., log-likelihood).
+#' models with \code{mirt} and aggregating the results (e.g., log-likelihood).
 #' However, constrains may be imposed across groups by invoking various \code{invariance} keywords and
 #' \code{constrain = ...} arguments, by inputting user specified design matrix from
 #' \code{mod2values} or from passing \code{pars = 'values'}, or by supplying a \code{constrain} list
@@ -18,9 +18,9 @@
 #' fitted,MultipleGroupClass-method
 #' @param data a \code{matrix} or \code{data.frame} that consists of
 #' numerically ordered data, with missing data coded as \code{NA}
-#' @param model an object or named list of objects returned from \code{confmirt.model()} declaring how
+#' @param model an object or named list of objects returned from \code{mirt.model()} declaring how
 #' the factor model is to be estimated. The names of the list input must correspond to the unique values
-#' in the \code{group} variable. See \code{\link{confmirt.model}} for more details
+#' in the \code{group} variable. See \code{\link{mirt.model}} for more details
 #' @param group a character vector indicating group membership
 #' @param invariance a character vector containing the following possible options:
 #' \describe{
@@ -44,13 +44,13 @@
 #' @param upper initial (or fixed) upper bound parameters for 4-PL model. Can be
 #' entered as a single value to assign a global upper bound parameter or may be entered as a
 #' numeric vector corresponding to each item
+#' @param accelerate see \code{\link{mirt}} for more details
 #' @param SE logical; estimate the information matrix for standard errors?
 #' @param SE.type see \code{\link{mirt}} for more details
 #' @param verbose logical; display iteration history during estimation?
 #' @param draws the number of Monte Carlo draws to estimate the log-likelihood
 #' @param quadpts the number of quadratures to be used per dimensions when \code{method = 'EM'}
 #' @param calcNull logical; calculate the Null model for fit statics (e.g., TLI)?
-#' @param cl a cluster object from the \code{parallel} package
 #' @param method a character indicating whether to use the EM (\code{'EM'}) or the MH-RM
 #' (\code{'MHRM'}) algorithm
 #' @param type type of plot to view; can be \code{'info'} to show the test
@@ -91,18 +91,18 @@
 #' }
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso
-#' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{confmirt.model}}, \code{\link{mirt}},
-#' \code{\link{confmirt}}, \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}},
+#' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{mirt.model}}, \code{\link{mirt}},
+#' \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}},
 #' \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}},
 #' \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
 #' \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
-#' \code{\link{read.mirt}}, \code{\link{simdata}}, \code{\link{createItem}}
+#' \code{\link{read.mirt}}, \code{\link{simdata}}, \code{\link{createItem}}, \code{\link{mirtCluster}}
 #' @keywords models
 #' @usage
 #' multipleGroup(data, model, group, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SE.type = 'SEM',
 #' invariance = '', pars = NULL, method = 'EM', constrain = NULL,
 #' parprior = NULL, calcNull = TRUE, draws = 5000, quadpts = NULL, grsm.block = NULL, rsm.block = NULL,
-#' key = NULL, cl = NULL, technical = list(), verbose = TRUE, ...)
+#' key = NULL, technical = list(), accelerate = TRUE, verbose = TRUE, ...)
 #'
 #' \S4method{coef}{MultipleGroupClass}(object, digits = 3, verbose = TRUE, ...)
 #'
@@ -130,11 +130,10 @@
 #' dataset2 <- simdata(a, d, N, itemtype, mu = .1, sigma = matrix(1.5))
 #' dat <- rbind(dataset1, dataset2)
 #' group <- c(rep('D1', N), rep('D2', N))
-#' models <- confmirt.model('F1 = 1-15')
+#' models <- mirt.model('F1 = 1-15')
 #'
 #' mod_configural <- multipleGroup(dat, models, group = group) #completely separate analyses
 #'
-#' # prev.mod can save precious iterations and help to avoid local minimums
 #' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes')) #equal slopes
 #' #equal intercepts, free variance and means
 #' mod_scalar2 <- multipleGroup(dat, models, group = group, 
@@ -210,7 +209,7 @@
 #' group <- c(rep('D1', N), rep('D2', N))
 #'
 #' #group models
-#' model <- confmirt.model()
+#' model <- mirt.model()
 #'    F1 = 1-5
 #'    F2 = 6-10
 #'    F3 = 11-15
@@ -257,8 +256,8 @@
 #' dataset2 <- simdata(a, d, N, itemtype, mu = .1, sigma = matrix(1.5))
 #' dat <- rbind(dataset1, dataset2)
 #' group <- c(rep('D1', N), rep('D2', N))
-#' models <- confmirt.model('F1 = 1-15')
-#' models2 <- confmirt.model('
+#' models <- mirt.model('F1 = 1-15')
+#' models2 <- mirt.model('
 #'    F1 = 1-10
 #'    F2 = 10-15')
 #'
@@ -282,7 +281,7 @@ multipleGroup <- function(data, model, group, itemtype = NULL, guess = 0, upper
                           SE = FALSE, SE.type = 'SEM', invariance = '', pars = NULL,
                           method = 'EM', constrain = NULL, parprior = NULL, calcNull = TRUE,
                           draws = 5000, quadpts = NULL, grsm.block = NULL, rsm.block = NULL,
-                          key = NULL, cl = NULL, technical = list(), verbose = TRUE, ...)
+                          key = NULL, technical = list(), accelerate = TRUE, verbose = TRUE, ...)
 {
     Call <- match.call()
     invariance.check <- invariance %in% c('free_means', 'free_varcov')
@@ -293,7 +292,7 @@ multipleGroup <- function(data, model, group, itemtype = NULL, guess = 0, upper
                       pars=pars, constrain=constrain, SE=SE, grsm.block=grsm.block,
                       parprior=parprior, quadpts=quadpts, method=method, rsm.block=rsm.block,
                       technical = technical, verbose = verbose, calcNull=calcNull,
-                      SE.type = SE.type, cl=cl, key=key, ...)
+                      SE.type = SE.type, key=key, accelerate=accelerate, draws=draws, ...)
     if(is(mod, 'MultipleGroupClass'))
         mod@Call <- Call
     return(mod)
