@@ -9,7 +9,7 @@ NumericMatrix polyOuter(NumericMatrix, NumericVector,
 	NumericVector, NumericVector);
 
 NumericVector itemTrace(NumericVector, const double *, 
-        NumericMatrix, const double *, const double *, const double *, NumericVector);
+        NumericMatrix, const double *, const double *, NumericVector);
 
 #endif 
 
